#!/usr/bin/python
# -*- coding: utf-8 -*-
#"""
#KOPR 1.0
#"""

import os
import ssl
from math import *
import datetime
import koprdial
import koprclass
import comephem
#import cr2fits
from astropy.io import fits
import urllib.request, urllib.error, urllib.parse
from PyQt5 import QtCore, QtGui, QtWidgets

ctx = ssl.create_default_context()
ctx.check_hostname = False
ctx.verify_mode = ssl.CERT_NONE


def cfile(destig):
    return './comet-data/'+destig.replace('/','').replace(' ','')+'.dat'

def exclude(observers):
    fopen = open('./config-files/Excluded.dat', 'r')
    excluded = fopen.read().replace('\n','').replace(' ','').split(',')
    for item in observers:
      if item[0] in excluded:
        item[1] = 0
    
    return observers

def checkname(fname, text = ''):
    if not os.path.exists(fname):
      fopen = open(fname, 'w')
      fopen.write(text)
      fopen.close()

def checkcomet(fname, text = ''):
    if not os.path.exists(fname) or os.path.getsize(fname) == 0:
      fopen = open(fname, 'w')
      try:
        response = urllib.request.urlopen('https://kopr.astro.cz/data/CometData.dat', context=ctx)
        text = response.read().decode('UTF-8')
      except Exception as e:
        print(e)
        if 'Connection timed out' in str(e):
          koprdial.InfoDialog(1,'Connection timed out', 'Connection timed out to http://kopr.astro.cz')
      fopen.write(text)
      fopen.close()

def filecheck(missing = ''):
    if not os.path.exists('./comet-data/'):
      os.makedirs('./comet-data/')
    if not os.path.exists('./obs-data/'):
      os.makedirs('./obs-data/')
    if not os.path.exists('./config-files/'):
      os.makedirs('./config-files/')
      missing = 'all'
    # main config file
    defaults=[ 'ICQxx;first name;second name', 
               '5;2;1;60000;3;2;7;100;0;0;0', 
               ';;;;', 
               ';;;;;',
               ';;',
               ';;;;;;;;']
    checkname('./config-files/settings.dat', '\n'.join(defaults) )
    # configuration data
    checkname('./config-files/location.dat')
    checkname('./config-files/telescope.dat')
    checkname('./config-files/eyepiece.dat')
    checkname('./config-files/camera.dat')
    # important files
    checkname('./config-files/CustomElements.dat')
    checkname('./config-files/Excluded.dat')
    checkname('./config-files/MagParams.dat')
    checkcomet('./config-files/CometData.dat')

    return missing

def queryVizier(star, refsrc, flt = 'C', limit = 0.3):
    star = star.replace(' ','%20')
    starra, starde, starmag, starbv = '0.','0.','0.','0.'
    try:
      if refsrc == 'TK':
        link =  'http://vizier.u-strasbg.fr/viz-bin/asu-txt?-source=I/259/tyc2&-c='+star+'&-out=_r&-out=_RAJ2000&-out=_DEJ2000&-out=VTmag&-out=BTmag&-sort=_r'
        response = urllib.request.urlopen(link, context=ctx)
        print('URL request',link)
        html = [x.decode('UTF-8') for x in response.readlines()]
        output = []
        for line in html:
            if not (line.startswith('#') or line.startswith(' ') or line.startswith('\n') or line.startswith('------') or line.startswith('_r') or line.startswith('rcmin')):
                output.append(line.split())
        starra, starde, starmag, starb = output[0][1], output[0][2], output[0][3], output[0][4]

        starbv = str( float(starb) - float(starmag) )

      elif refsrc == 'AQ':
        if flt == 'V':
            fltsrc = 'Vmag'
        elif flt == 'k':
            fltsrc = 'r\'mag'
        elif flt == 'D':
            fltsrc = 'Bmag'
        else:
            fltsrc = 'Vmag'
            
        link = 'http://vizier.u-strasbg.fr/viz-bin/asu-txt?-source=II/336/apass9&-c='+star+'&-out=_r&-out=_RAJ2000&-out=_DEJ2000&-out='+fltsrc+'&-out=B-V&-sort=_r'
        print('URL request', link)
        response = urllib.request.urlopen(link , context=ctx)
        html = [x.decode('UTF-8') for x in response.readlines()]
        mags, dist = [], {}
        for line in html:
            if not (line.startswith('#') or line.startswith(' ') or line.startswith('\n') or line.startswith('------') or line.startswith('_r') or line.startswith('rcmin')):
                if float(line.split()[0]) < limit and len(line.split()) == 5:
                    mags.append(float(line.split()[3]))
                    dist[float(line.split()[3])] = line.split()
        if len(dist) != 0:
            starra, starde, starmag, starbv = dist[min(mags)][1], dist[min(mags)][2], dist[min(mags)][3], dist[min(mags)][4]
        else:
            koprdial.InfoDialog(1,'Star not found','No star was found near position on Vizier','See details for Vizier query','http://vizier.u-strasbg.fr/viz-bin/asu-txt?-source=II/336/apass9&-c='+star+'&-out=_r&-out=_RAJ2000&-out=_DEJ2000&-out='+fltsrc+'&-out=B-V&-sort=_r')

    except Exception as e:
      if 'Connection timed out' in str(e):
        koprdial.InfoDialog(1,'Connection timed out', 'Connection timed out to http://vizier.u-strasbg.fr')
    
    return float(starra), float(starde), starmag, starbv

def UpdateCCDDef(defaults):
    conf_file = open('./config-files/config.dat', 'r', encoding="utf-8")
    for line in conf_file.readlines():
        obscode, fname, lname, rest = line.split(',')
        break
    conf_file.close()

    conf_file = open('./config-files/config.dat', 'w', encoding="utf-8")
    conf_file.write( ','.join( [obscode, fname, lname, ';'.join(defaults) ] ).replace('\n','') ) 
    conf_file.close()

def InitFile():
    conf_file = open('./config-files/config.dat', 'r', encoding="utf-8")
    for line in conf_file.readlines():
      if len(line.split(',')) == 3:
        obscode, fname, lname = line.split(',')
        defaults = ''
      elif len(line.split(',')) == 4:
        obscode, fname, lname, defaults = line.split(',')
      break
    conf_file.close()
    
    if not defaults:
      defaults = [ '5', '2', '1', '60000', '3', '2', '7', '100' ]
      conf_file = open('./config-files/config.dat', 'w', encoding="utf-8")
      conf_file.write( ','.join( [obscode, fname, lname, ';'.join(defaults) ] ).replace('\n','') ) 
    else:
      defaults = defaults.split(';')

    loc_file = open('./config-files/location.dat', 'r', encoding="utf-8")
    locations = loc_file.readlines()
    loc_file.close()

    tel_file = open('./config-files/telescope.dat', 'r', encoding="utf-8")
    telescopes = tel_file.readlines()
    tel_file.close()

    eye_file = open('./config-files/eyepiece.dat', 'r', encoding="utf-8")
    eyepieces = eye_file.readlines()
    eye_file.close()

    ccd_file = open('./config-files/camera.dat', 'r', encoding="utf-8")
    cameras = ccd_file.readlines()
    ccd_file.close()
    
    return obscode, fname, lname, locations, telescopes, eyepieces, cameras, defaults

def loadcobsfilecom():
    comet_file = open('./config-files/comet-cobs.txt', 'r', encoding="utf-8") #nahrajeme pozorovani
    cometlist = []
    for line in comet_file.readlines():
       comet, dtime, mag = line.split(',')
       if comet not in cometlist:
         cometlist.append(comet)
    cometlist.sort(reverse=False)

    return cometlist

def loadmpccomet(destig, kometa):
    perihelions, mpcobs = [], koprclass.icqcode()
    count, obs, obslist = 0, '', []

    try:
      link = 'http://www.minorplanetcenter.net/db_search/show_object?utf8=%E2%9C%93&object_id='+destig.replace('/','%2F').replace(' ','+')
      print('URL request'), link
      response = urllib.request.urlopen(link, context=ctx)
      html = [x.decode('UTF-8') for x in response.readlines()]
      for line in html:
        if ('<td class="ff">' in line and count == 0) or (0 < count < 6) :
          count += 1
          if count == 1: 
            mpcobs.date = line.strip().replace('<td class="ff">','').replace('</td>','').replace(' ','/')    
          elif count == 4: 
            inpt = line.strip().replace('<td class="ff rj">','').replace('</td>','')
            if 'T' in inpt:
              mpcobs.mag, mpcobs.otype = float(inpt.split()[0]), 'mpct'
            elif 'N' in inpt:
              mpcobs.mag, mpcobs.otype = float(inpt.split()[0]), 'mpcn'
            else:
              mpcobs.otype = 'nomag'
          elif count == 5: 
            inpt = line.strip().replace('<td>','').replace('</td>','')
            mpcobs.obscode, mpcobs.obsnote = inpt.split(' &ndash; ')[0], inpt.split(' &ndash; ')[1]
            mpcobs.comet = destig
            if mpcobs.otype != 'nomag':
              peridate = kometa.FindElements(mpcobs.date)[2]
              if peridate[0:4] not in perihelions:
                perihelions.append(peridate[0:4])
              obslist.append(mpcobs.build_icq()[0])
        if count == 6:
          mpcobs = koprclass.icqcode()
          count = 0

    except Exception as e:
      if 'Connection timed out' in str(e):
        koprdial.InfoDialog(1,'Connection timed out', 'Connection timed out to http://www.minorplanetcenter.net')
  
    return perihelions, obslist 

def LoadValidObsList(destig):
    obs_lines = []
    fname = './comet-data/'+destig.replace('/','').replace(' ','')+'.dat'
    if not os.path.exists(fname):
      results = ObsResults( loadcobscomet(destig) , destig, [])

    comet_file = open(fname, 'r', encoding="utf-8") #nahrajeme pozorovani
    obs_lines = [ x for x in comet_file.readlines() if x[28:32].replace('.', '').replace(' ', '').isnumeric() ]
    comet_file.close()

    return obs_lines

def ObsResults(filtobs, destig, obs_array ):
    results = []
    for line in filtobs:
      try:
        icq = koprclass.icqcode()
        line = line.replace('\n','')
        icq.decode_icq(line)
        zoom, coma = icq.get_zoom(), icq.get_coma() 
       
        try: # generate signature of observation 
          float(icq.apert)
        except ValueError:
          icq.apert = 0
        signature = ' '.join( [icq.comet, icq.obscode, str(timecvt(icq.date)), icq.ttype, str(float(icq.apert)), icq.method, str(zoom), str(coma)] ) 

        if signature in [x[0] for x in obs_array]: # check if signature exist and append observation
          index = [x[0] for x in obs_array].index(signature)
          obs_array[index][1].append(coma)
          obs_array[index][2].append(line)  
        else: 
          obs_array.append( [signature, [coma], [line]] )
      except AttributeError:
        print('AttributeError: lenght = ',len(line),' line: ',line)
            
    for line in obs_array: # update list of observations
      if len(line[1]) > 1:
        results.append(line[2][line[1].index(min(line[1]))]+'\n')
      else: 
        results.append(line[2][0]+'\n')

    obs_file = open(cfile(destig), 'w', encoding="utf-8") # update dat
    for line in results:
      obs_file.write(line)
    obs_file.close()
    
    return results

def ObsFileUpdate( location, destig, filtobs ):
    if location != 'local':
        oldobs = [] #zachrana starych dat
        try: 
          obs_file = open('./comet-data/'+destig.replace('/','').replace(' ','')+'.dat', 'r', encoding="utf-8")
          obs_lines = obs_file.readlines()
          for obs in obs_lines:
            if obs[68:72].startswith('MPC') and location == 'cobs':
              oldobs.append(obs.replace('\n',''))
            elif location == 'mpc':
              line = obs.replace('\n','')
              if line not in filtobs:
                oldobs.append(line)
          obs_file.close()
        except:
          pass        
        
        obs_file = open('./comet-data/'+destig.replace('/','').replace(' ','')+'.dat', 'w', encoding="utf-8") # update dat
        for line in oldobs:
            obs_file.write(line+'\n')
        for line in filtobs:
            obs_file.write(line+'\n')
        obs_file.close()

    return LoadValidObsList(destig)

def loadcobscomet(destig, beg='', end=''):
    destig = destig.replace(' ','%20')
    beg, end = beg.replace('/','-'), end.replace('/','-')
    filtobs = []
    try:
      if not beg and not end:
        response = urllib.request.urlopen('https://cobs.si/api/obs_list.api?format=icq&des='+destig, context=ctx)        
        print('URL request', 'https://cobs.si/api/obs_list.api?format=icq&des='+destig)
      else:
        response = urllib.request.urlopen('https://cobs.si/api/obs_list.api?format=icq&des='+destig+'&from_date='+beg+'&to_date='+end, context=ctx) #2016/09/02%2000:00
        print('URL request', 'https://cobs.si/api/obs_list.api?format=icq&des='+destig+'&from_date='+beg+'&to_date='+end)

      html = [x.decode('UTF-8') for x in response.readlines()]
      for line in html:
        try:
          float(line[28:32])
          filtobs.append(line)
        except Exception as e:
          pass
#          print('loadcobscomet ',e)

    except Exception as e:
      print('URL failed', 'https://cobs.si/api/obs_list.api?format=icq&des='+destig)
      print(e)
      if 'Connection timed out' in str(e):
        koprdial.InfoDialog(1,'Connection timed out', 'Connection timed out to https://.cobs.si')

    if len(filtobs) > 2300:
      for i in range(2,10):
        filtobs0 = []
        try:
          if not beg and not end:
            response = urllib.request.urlopen('https://cobs.si/api/obs_list.api?format=icq&des='+destig+'&page='+str(i), context=ctx)        
            print('URL request', 'https://cobs.si/api/obs_list.api?format=icq&des='+destig+'&page='+str(i))
          else:
            response = urllib.request.urlopen('https://cobs.si/api/obs_list.api?format=icq&des='+destig+'&from_date='+beg+'&to_date='+end+'&page='+str(i), context=ctx) #2016/09/02%2000:00
            print('URL request', 'https://cobs.si/api/obs_list.api?format=icq&des='+destig+'&from_date='+beg+'&to_date='+end+'&page='+str(i))

          html = [x.decode('UTF-8') for x in response.readlines()]
          for line in html:
            try:
              float(line[28:32])
              filtobs0.append(line)
            except:
              pass
        except Exception as e:
          print('URL failed', 'https://cobs.si/api/obs_list.api?format=icq&des='+destig)
          if 'Connection timed out' in str(e):
            koprdial.InfoDialog(1,'Connection timed out', 'Connection timed out to https://.cobs.si')

        print('filtobs0', len(filtobs0))
        if len(filtobs0) > 0:
          filtobs += filtobs0
        else:
          break

    return filtobs

def loadcometid(destig):
    orb_file = open('./config-files/Elements.txt', 'r', encoding="utf-8")
    for line in orb_file.readlines():
       if DestigFromName(line[102:159]) == destig:
          eline = line
    orb_file.close()
    
    orb_file = open('./config-files/CustomElements.dat', 'r', encoding="utf-8")
    for line in orb_file.readlines():
       if DestigFromName(line[102:159]) == destig:
          eline = line
    orb_file.close()
    
    try:
      return eline
    except UnboundLocalError:
      koprdial.InfoDialog(1,'Elements not found', 'Elements of comet not found')
      return ''

def CometName(destig):
    name = ''
    orb_file = open('./config-files/Elements.txt', 'r', encoding="utf-8")
    for line in orb_file.readlines():
       if DestigFromName(line[102:159]) == destig:
          name = line[102:159].strip()
    orb_file.close()
    
    orb_file = open('./config-files/CustomElements.dat', 'r', encoding="utf-8")
    for line in orb_file.readlines():
       if DestigFromName(line[102:159]) == destig:
          name = line[102:159].strip()
    orb_file.close()

    return name 

def DestigFromName(name):
    name = name.strip() #    print 'original name ... ', name
    strpos = name.find('/', 0, 7)
    if strpos == 1:
        destig = name.split(' ')[0]+' '+name.split(' ')[1]
    elif strpos > 1 or strpos == -1:
        destig = name.split('/',1)[0]
    else:
        print('Failed .................')#    print 'returend destig name ... ', destig
    return destig

def DestigFromICQ(name):
    try:
      int(name[0:3])
    except ValueError:
      isnumeric = False
    else:
      isnumeric = True

    if isnumeric:
      destig = (name[0:3]+'P').replace(' ','')
    else: 
      if name[2] == 'P':
        prefix = 'P/'
      else:
        prefix = 'C/'
      destig = prefix + name[3:7] + ' ' + name[7:10].replace(' ','')

    return destig

def updatecobs():
    ep = datetime.datetime(1970,1,1,0,0,0)
    now = (datetime.datetime.now()- ep).total_seconds()
    try:
      lastmod = os.path.getmtime('./config-files/comet-cobs.txt')
    except OSError:
      lastmod = now - 86401

    if now - lastmod > 86400 or os.path.getsize('./config-files/comet-cobs.txt') == 0:
      msgBox = QtWidgets.QMessageBox(QtWidgets.QMessageBox.Information, 'COBS update', 'Do you want update latest COBS magnitudes?', QtWidgets.QMessageBox.Ok | QtWidgets.QMessageBox.Cancel)
      returnValue = msgBox.exec()
      if returnValue == QtWidgets.QMessageBox.Ok:
        try:
          start = datetime.datetime.now() - datetime.timedelta(days = 30)
          beg = '-'.join( [str(start.year),str(start.month),str(start.day)] )
          response = urllib.request.urlopen('https://cobs.si/api/obs_list.api?format=icq&from_date=' + beg, context=ctx) 
          html = [x.decode('UTF-8') for x in response.readlines()]
          comet_file = open('./config-files/comet-cobs.txt', 'w', encoding="utf-8")
          for line in html:
            try:
              float(line[28:32])
              obs = koprclass.icqcode()
              obs.decode_icq(line)
              comet_file.write(','.join( [ DestigFromICQ(obs.comet), obs.date, str(obs.mag) ] ) + '\n')
            except:
              pass
          comet_file.close()
        except Exception as e:
          if 'Connection timed out' in str(e):
            koprdial.InfoDialog(1,'Connection timed out', 'Connection timed out to https://cobs.si/cobs/recent/')

def degconvert(dinp):
    if dinp.find('d') != -1 and dinp.find('m') != -1 and dinp.find('s') != -1:
        deg,remn = dinp.split('d')
        arcmin,arcsec = remn.split('m')
        deg, arcmin, arcsec = float(deg), float(arcmin), float(arcsec.replace('s',''))
        deg = deg + arcmin/60. + arcsec/3600.
    elif dinp.find('d') != -1 and dinp.find('m') != -1 and dinp.find('s') == -1:
        deg,arcmin = dinp.split('d')
        deg, arcmin = float(deg), float(arcmin.replace('m',''))
        deg = deg + arcmin/60.
    elif dinp.find('d') != -1 and dinp.find('m') == -1 and dinp.find('s') == -1:
        deg = float(dinp.replace('d',''))
    else:
        deg = float(dinp)
    return deg

def timecvt(datedd): # converting ephem format to datetime
    year = int(datedd.split('/')[0])
    month = int(datedd.split('/')[1])
    if '.' in datedd:
        day = int(datedd.split('/')[2].split('.')[0])
        hour = float('0.'+datedd.split('/')[2].split('.')[1])*24.
        minute = (float(hour) - int(hour))*60.
    else:
        day = int(datedd.split('/')[2])
        hour = 0
        minute = 0
    try:      
      if int(hour)<24:
        date = datetime.datetime(year,month,day,int(hour),int(minute))
      else:
        date = datetime.datetime(year,month,day,int(hour)-1,int(minute)+59,59)
    except ValueError:
      date = datetime.datetime(year,month,day-1,23,59,59)

    return date

def timecvt2(datedd): # converting ephem format with hours to datetime
    try:
      if int(datedd.split()[1].split(':')[0])<24:
        outputdate = datetime.datetime(int(datedd.split()[0].split('/')[0]), int(datedd.split()[0].split('/')[1]), int(datedd.split()[0].split('/')[2]), int(datedd.split()[1].split(':')[0]), int(datedd.split()[1].split(':')[1]))
      else:
        outputdate = datetime.datetime(int(datedd.split()[0].split('/')[0]), int(datedd.split()[0].split('/')[1]), int(datedd.split()[0].split('/')[2]), int(datedd.split()[1].split(':')[0])-1,59,59)      
    except ValueError:
      outputdate = datetime.datetime(int(datedd.split()[0].split('/')[0]), int(datedd.split()[0].split('/')[1]), int(datedd.split()[0].split('/')[2])-1, 23,59,59)
    return outputdate

def cvt2time(datedd): # converting datetime to ephem
    time = datedd.day + datedd.hour/24. + datedd.minute/1440. + datedd.second/86400.
    date = '{:4d}/{:2d}/{:5.2f}'.format(datedd.year, datedd.month, round(time,3) ) 
    return date

def time2deltat(datedd): # converting delta days to integer
    return float(datedd.days) + float(datedd.seconds)/86400.

def LoadPersonalCoef():
    dictionary = {}
    comet_file = open('./config-files/Personal-coef.txt', 'r', encoding="utf-8")
    for line in comet_file.readlines():
        dictionary[line.split('=')[0]] = float(line.split('=')[1])
    comet_file.close()
    return dictionary

def GetTeldata(telescopes):
    ttype, tfocl, taper = '', 0, 0
    if telescopes == 'Naked Eye':
        ttype, tfocl, taper = 'E', 0, 0
    else:
        tel_file = open('./config-files/telescope.dat', 'r', encoding="utf-8")
        for line in tel_file.readlines():
            ltname,lttype,ltaper,ltfocl = line.split(',')
            if telescopes == ltname:
                tname,ttype,taper,tfocl = line.split(',')
                tfocl, taper = float(tfocl), float(taper)
        tel_file.close()
    return ttype, tfocl, taper

def GetEyedata(eyepieces):
    ename,efocl,efov = '', 0, 0
    eye_file = open('./config-files/eyepiece.dat', 'r', encoding="utf-8")
    for line in eye_file.readlines():
        lename,lefocl,lefov = line.split(',')
        if eyepieces == lename:
            ename,efocl,efov = line.split(',')[0], float(line.split(',')[1]), float(line.split(',')[2])  
    eye_file.close()
    return efocl,efov 

def CometUpdate():    
    ep = datetime.datetime(1970,1,1,0,0,0)
    now = (datetime.datetime.now()- ep).total_seconds()

    try:
      lastmod = os.path.getmtime('./config-files/Elements.txt')
    except OSError:
      lastmod = now - 86401

    if now - lastmod > 86400 or os.path.getsize('./config-files/Elements.txt') == 0:
      try:
        print('updating comet orbit elements ...')
        response = urllib.request.urlopen('https://minorplanetcenter.net/iau/Ephemerides/Comets/Soft00Cmt.txt', context=ctx)
        html = [x.decode('UTF-8') for x in response.readlines()]
        comet_file = open('./config-files/Elements.txt', 'w', encoding="utf-8")
        for line in html:
           comet_file.write(line)
        comet_file.close()   
        
        if os.path.getsize('./config-files/Elements.txt') > 50000: # test correct file size
          print('orbit elements updated')
        else:
          print('ERROR: orbit elements update failed')

      except Exception as e:
        if 'Connection timed out' in str(e):
          koprdial.InfoDialog(1,'Connection timed out', 'Connection timed out to https://minorplanetcenter.net')
        else:
          print(e)

def ObsUpdate(output):
    if '' in output.split(','): 
        koprdial.InfoDialog(1, 'Error', 'Fill all required fields!')
    else:
        rest = ''
        conf_file = open('./config-files/config.dat', 'r', encoding="utf-8")
        for line in conf_file.readlines():
          if len(line.split(',')) == 4:
            rest = line.split(',')[3]
          break
        conf_file.close()

        if not rest:
          defaults = [ '5', '2', '1', '60000', '3', '2', '7', '100' ]
          rest = ';'.join(defaults)

        conf_file = open('./config-files/config.dat', 'w')
        conf_file.write( output + ',' +rest.replace('\n','') ) 
        conf_file.close()
        koprdial.InfoDialog(0, 'Success', 'Observer saved')

def ItemRem(oldlist, typ, count = 0):
    newlist, step = [], 0
    if typ == 'loc':
        f, infotext = open('./config-files/location.dat', 'w', encoding="utf-8"), 'Location was removed'
    elif typ == 'tel':
        f, infotext = open('./config-files/telescope.dat', 'w', encoding="utf-8"), 'Telescope was removed'
    elif typ == 'eye':
        f, infotext = open('./config-files/eyepiece.dat', 'w', encoding="utf-8"), 'Eyepiece was removed'
    elif typ == 'cam':
        f, infotext = open('./config-files/camera.dat', 'w', encoding="utf-8"), 'Camera was removed'

    for line in oldlist:
        if count != step:
            newlist.append(line)
            f.write(line)
        step += 1  
    f.close()

    koprdial.InfoDialog(0, 'Success', infotext)
    return newlist

########## to be deleted start
def showdialogCom(destig, date = 'none', time ='none'):
    critical, errcount, errormsg, errordetmsg = 0, 0, '', ''
    
    orb_file = open('./config-files/Elements.txt', 'r', encoding="utf-8")
    f = orb_file.read()
    orb_file.close()

    orb_file = open('./config-files/CustomElements.dat', 'r', encoding="utf-8")
    s = orb_file.read()
    orb_file.close()

    if destig == '':
        critical = 1
        errcount += 1
        errormsg = errormsg + 'Comet error\n'
        errordetmsg = errordetmsg + '* Insert comet name first\n'
    elif destig not in f and destig not in s:
        critical = 1
        errcount += 1
        errormsg = errormsg + 'Comet not found\n'
        errordetmsg = errordetmsg + '* Comet not found in database\n'
    if date == '' or time == '':
        errcount += 1
        critical = 1
        errormsg = errormsg + 'Date and time error'
        errordetmsg = errordetmsg + '* Date or Time input is missing'

    return critical, errcount, errormsg, errordetmsg
########## to be deleted start

def showdialogCal(refstars, coma, tail, pa):
    critical, errcount, errormsg, errordetmsg = 0, 0, '', ''

    if len(refstars) < 2:
        errcount += 1
        critical = 1
        errormsg = errormsg + 'Reference stars error\n'
        errordetmsg = errordetmsg + '* Input reference stars first\n'
    if coma == '':
        errcount += 1
        critical = 1
        errormsg = errormsg + 'Coma error\n'
        errordetmsg = errordetmsg + '* Coma diameter is missing\n'
    if pa == '' and tail != '':
        errcount += 1
        critical = 1
        errormsg = errormsg + 'Tail p.a. error\n'
        errordetmsg = errordetmsg + '* Tail is stated, but position angle is missing\n'
    
    return critical, errcount, errormsg, errordetmsg 

def showdialogCCD(refstars, apert, tail, pa, exptime, expnr ):
    critical, errcount, errormsg, errordetmsg = 0, 0, '', ''

    count = 0
    for star in refstars:
        if star[0] == 'c':
            count += 1
    if count == 0:
        errcount += 1
        critical, errormsg = 1, 'Comet data missing\n'
        errordetmsg = '* Measure comet first\n'

    count = 0
    for star in refstars:
        if star[0] == 'o':
            count += 1
    if count < 3:
        errcount += 1
        critical, errormsg = 1, 'Star data missing\n'
        errordetmsg = '* Measure at least 3 stars\n'

    if pa == '' and tail != '':
        errcount += 1
        critical = 1
        errormsg = errormsg + 'Tail p.a. error\n'
        errordetmsg = errordetmsg + '* Tail is stated, but position angle is missing\n'
    if apert == '':
        errcount += 1
        critical = 1
        errormsg = errormsg + 'Aperture error\n'
        errordetmsg = errordetmsg + '* Aperture radius is missing\n'
    if exptime == '':
        errcount += 1
        critical = 1
        errormsg = errormsg + 'Exposure time error\n'
        errordetmsg = errordetmsg + '* Exposure time of coadded image is missing\n'
    if expnr == '':
        errcount += 1
        critical = 1
        errormsg = errormsg + 'Exposure number error\n'
        errordetmsg = errordetmsg + '* Exposure number of coadded image is missing\n'
    
    return critical, errcount, errormsg, errordetmsg 

def showdialogOut( cometalt, cometmag, moonalt, moonp, sunalt, slope, std_err, extinction, xlist = [] ):                                                                                       
    critical, warning, errcount, errormsg, errordetmsg = 0, 0, 0, '', ''                                                                                             
                                                                                                                                                                     
    if xlist != []:
        if (min(xlist) < 0 and max(xlist) < 0) or (min(xlist) > 0 and max(xlist) > 0):                                                                                   
            errcount += 1                                                                                                                                                
            critical = 1                                                                                                                                                 
            errormsg = errormsg + 'Estimate error\n'                                                                                                                     
            errordetmsg = errordetmsg + '* you need at least one brighter and one fainter star to estimate comet magnitude\n'                                            
    if cometalt <0 :                                                                                                                                                 
        errcount += 1                                                                                                                                                
        critical = 1                                                                                                                                                 
        errormsg = errormsg + 'Comet bellow horizon\n'                                                                                                               
        errordetmsg = errordetmsg + '* Comet is bellow horizon, please check time of observation\n'                                                                  
    elif cometalt <10 and extinction == 'No Extinction':                                                                                                             
        errcount += 1                                                                                                                                                
        critical = 1                                                                                                                                                 
        errormsg = errormsg + 'Extinction missing\n'                                                                                                                 
        errordetmsg = errordetmsg + '* Extinction correction is missing for comet less than 10 degree over horizon\n'                                                
    elif sunalt > 0 and cometmag > -4:                                                                                                                               
        errcount += 1                                                                                                                                                
        critical = 1                                                                                                                                                 
        errormsg = errormsg + 'Daylight observation\n'                                                                                                               
        errordetmsg = errordetmsg + '* You can\'t observer such faint comet during daylight.\n'                                                                      
    elif sunalt > -6 and cometmag > 5:                                                                                                                               
        errcount += 1                                                                                                                                                
        critical = 1                                                                                                                                                 
        errormsg = errormsg + 'Civil twilight\n'                                                                                                                     
        errordetmsg = errordetmsg + '* Comet is too faint to be observed during civil twilight\n'                                                                    
    elif slope < 0:                                                                                                                                                  
        errcount += 1                                                                                                                                                
        critical = 1                                                                                                                                                 
        errormsg = errormsg + 'Negative slope\n'                                                                                                                     
        errordetmsg = errordetmsg + '* Your fainter stars are brighter than your brighter stars, check your Argelander degree sign\n'                                
    if critical == 0:                                                                                                                                                
        if sunalt > -18:                                                                                                                                             
            errcount += 1                                                                                                                                            
            warning = 1                                                                                                                                              
            errormsg = errormsg + 'Twilight observation\n'                                                                                                           
            errordetmsg = errordetmsg + '* Comet was not observed during full night, the estimate may be low quality\n'                                              
        elif moonalt > 5 and moonp > 0.1:                                                                                                                            
            errcount += 1                                                                                                                                            
            warning = 1                                                                                                                                              
            errormsg = errormsg + 'Moonlight\n'                                                                                                                      
            errordetmsg = errordetmsg + '* Comet was observed with Moon over horizon, the estimate may be low quality\n'                                             
        elif std_err > 0.1:                                                                                                                                          
            errcount += 1                                                                                                                                            
            warning = 1                                                                                                                                              
            errormsg = errormsg + 'Estimate precision\n'                                                                                                             
            errordetmsg = errordetmsg + '* The standard deviation in your estimate is too high\n'                                                                    
                                                                                                                                                                     
    return critical, warning, errcount, errormsg, errordetmsg                                                                                                        

def showdialogCmes( counter, apert, fnamec, fnames ):
    critical, warning, errcount, errormsg, errordetmsg = 0, 0, 0, '', ''                                                                                             

    if counter == 0 and apert == '':
        errcount += 1
        critical = 1
        errormsg = errormsg + 'Aperture error\n'
        errordetmsg = errordetmsg + '* Comet pperture in px is missing\n'
    if counter != 0 and apert == '':
        errcount += 1
        critical = 1
        errormsg = errormsg + 'Aperture error\n'
        errordetmsg = errordetmsg + '* Star aperture in px is missing\n'
    if (counter == 0 and fnamec == '') or (counter != 0 and fnamec == '' and fnames == ''):
        errcount += 1
        critical = 1
        errormsg = errormsg + 'Image missing'
        errordetmsg = errordetmsg + '* Load FITS image first'

    return critical, errcount, errormsg, errordetmsg 
        
def showdialogImg():
    critical, warning, errcount, errormsg, errordetmsg = 0, 0, 0, '', ''                                                                                             
    if btype == 'calc':
        orb_file = open('./config-files/Elements.txt', 'r', encoding="utf-8")
        if str(self.le1.text()) == '':
            critical = 1
            errcount += 1
            errormsg = errormsg + 'Comet error\n'
            errordetmsg = errordetmsg + '* Insert comet name first\n'
        elif str(self.le1.text()) not in orb_file.read():
            critical = 1
            errcount += 1
            errormsg = errormsg + 'Comet not found\n'
            errordetmsg = errordetmsg + '* Comet not found in database\n'
        orb_file.close()
        if (str(self.le2.text()) == '' or str(self.le3.text()) == ''):
            errcount += 1
            critical = 1
            errormsg = errormsg + 'Date and time error\n'
            errordetmsg = errordetmsg + '* Date or Time input is missing\n'
        errormsg1, errormsg2, errormsg3 = '', '', ''
        errordetmsg1, errordetmsg2, errordetmsg3 = '', '', ''
        aslist = list()
        for i in range(1,refstars+1):
            if str(self.asle[i].text()) == '':
                errcount += 1
                critical = 1
                errormsg1 = 'Magnitude error\n'
                errordetmsg1 = '* Measured relative magnitude is missing for one or more stars\n'
            if str(self.magle[i].text()) == '':
                errcount += 1
                critical = 1
                errormsg2 = 'Magnitude error\n'
                errordetmsg2 = '* Reference star magnitude is missing for one or more stars\n'
            if str(self.amagle[i].text()) == '' and refsrc == 'AQ' and obsmethod == 'k':
                errcount += 1
                critical = 1
                errormsg2 = 'Magnitude error\n'
                errordetmsg2 = '* Reference star magnitude is missing for one or more stars\n'
            if str(self.extle[i].text()) == '' and extinction != 'No Extinction':
                errcount += 1
                critical = 1
                errormsg3 = 'Altitude error\n'
                errordetmsg3 = '* Altitude is missing for one or more stars\n'
        errormsg = errormsg + errormsg1 + errormsg2 + errormsg3
        errordetmsg = errordetmsg + errordetmsg1 + errordetmsg2 + errordetmsg3

        if str(self.le4.text()) == '':
            errcount += 1
            critical = 1
            errormsg = errormsg + 'Aperture error\n'
            errordetmsg = errordetmsg + '* Aperture radius is missing\n'
        if str(self.le7.text()) == '':
            errcount += 1
            critical = 1
            errormsg = errormsg + 'Exposure time error\n'
            errordetmsg = errordetmsg + '* Exposure time of coadded image is missing\n'
        if str(self.le8.text()) == '':
            errcount += 1
            critical = 1
            errormsg = errormsg + 'Exposure number error\n'
            errordetmsg = errordetmsg + '* Exposure number of coadded image is missing\n'
        if str(self.le6.text()) == '' and str(self.le5.text()) != '':
            errcount += 1
            critical = 1
            errormsg = errormsg + 'Tail p.a. error\n'
            errordetmsg = errordetmsg + '* Tail is stated, but position angle is missing\n'
        
    tfocl, efocl, taper = float(tfocl), float(efocl), float(taper)
    if tfocl > 0 and ttype!='B':
        text = 'zoom: '+ str( round(tfocl/efocl,0) )+'x' 
    elif ttype == 'B':
        text = 'zoom: '+ str(tfocl)+'x' 
    else: 
        text = 'zoom'        
    output = 'Telescope aperture diameter: ' + str(round(taper/10,1)) + '-cm, ' + text
    return output

def ImageLoad(image_data = [], shape = 0, header = {}):
    try:
      envfile = open('./config-files/envdir.dat', 'r', encoding="utf-8")
      fdir = envfile.read()
      envfile.close()
    except IOError:
      fdir = './'

    fname = QtWidgets.QFileDialog.getOpenFileName( None, 'Open file', fdir, 'Image files (*.fits *.fit)' )

    if fname != '':
        fdir = str(fname).rsplit('/',1)[0]
        envfile = open('./config-files/envdir.dat', 'w', encoding="utf-8")
        envfile.write(fdir)
        envfile.close()

        hdulist = fits.open(fname[0])
        shape, header = len(hdulist[0].data.shape), hdulist[0].header

        #astrometry test
        print('astrometry test')
        from astropy.wcs import WCS
        try:
          proj = WCS(header)
        except ValueError:
          print('astrometry test failed, removing keywords XPIXELSZ, YPIXELSZ, CDELTM1, CDELTM2')
          hdulist.close()
          hdul = fits.open(fname[0], mode='update')
          h = hdul[0].header
          # remove keywords
          try:
            h.remove('XPIXELSZ')
            h.remove('YPIXELSZ')
            h.remove('CDELTM1')
            h.remove('CDELTM2')
            # save the header
            hdul.flush()
          except:
            print('removing keywords failed')
          # Close
          hdul.close()    
          hdulist = fits.open(fname[0])
          shape, header = len(hdulist[0].data.shape), hdulist[0].header
        #astrometry test end

        if shape == 2:
            image_data = hdulist[0].data
        elif shape == 3: 
            image_data = hdulist[0].data[1,:,:]
            koprdial.InfoDialog(0, 'Green chanel extracted', 'Green channel was extracted to measure comet from RGB image')

        hdulist.close()
    return fname, image_data, shape, header

#def MultipleImageLoad(image_data = [], shape = 0, header = {}):
#    fileList = QtWidgets.QFileDialog.getOpenFileNames( None, 'Open file', './', 'Image files (*.fits *.cr2)' )
#    if len(fileList) > 0:
#      for i, item in enumerate(fileList):
#        if item[-4:] == '.cr2' or item[-4:] == '.CR2': 
#          cr2 = cr2fits.cr2fits(str(item), 1)
#          dest = cr2.convert()
#          fileList[i] = dest
#
#    return fileList
      
def GetActualCurve(destig):
    elements, xlist, ylist, t, s = comephem.ephem(), [], [], [], []

    if not destig[0:1].isdigit():
        destig = destig.split()[0]+' '+destig.split()[1]

    orb_file = open('./config-files/Elements.txt', 'r', encoding="utf-8")
    for line in orb_file.readlines():
       if destig in line:
           elements.readdb(line)
           rest = line[90:102].split()
           h0, n = rest[0], rest[1]# Magnitude parameters
    orb_file.close()

    orb_file = open('./config-files/CustomElements.dat', 'r', encoding="utf-8")
    for line in orb_file.readlines():
       if destig in line:
           elements.readdb(line)
           rest = line[90:102].split()
           h0, n = rest[0], rest[1]# Magnitude parameters
    orb_file.close()

    comet_file = open('./config-files/comet-cobs.txt', 'r', encoding="utf-8") #nahrajeme pozorovani
    
    xdeltacorr = []
    for line in comet_file.readlines():
        comet, dtime, mag = line.split(',')
        if destig in comet:
            year, month, day = dtime.split('/')
            hour = int((float(day) - int(float(day)))*24)
            minute = int((hour - int(hour))*60)
            deltadate = datetime.datetime(int(year), int(month), int(float(day)), hour) - datetime.datetime.now()
            deltadate, rest, rest2 = str(deltadate).split(' ')
            elements.compute('/'.join( [year,month,str(int(float(day)))] ),':'.join( [str(int(hour)), str(minute)] ),['comet'])
            mag = float(mag.replace('\n',''))
            xlist.append(float(deltadate))
            ylist.append(mag)
            xdeltacorr.append( mag - 5*log10(elements.dist) - 10*log10(elements.r) )

    h0 = sum(xdeltacorr) / len(xdeltacorr)

    steps, datenow = int( (35 - (min(xlist)-5))*10), datetime.datetime.now()
    for i in range(steps):
        datenow = datetime.datetime.now() + datetime.timedelta(days= (i *0.1)+min(xlist))  
        date = str(datenow.strftime('%Y/%m/%d %H:%M'))
        elements.compute(date.split()[0],date.split()[1],['comet'])
        t.append( (min(xlist)-5) + i *0.1 )
        s.append( h0 + 5*log10(elements.dist) + 10*log10(elements.r) )
    
    magparams = 'absolute magnitude: '+str(round(h0, 2))+' mag, slope: 4.0'#+n

    return elements.name, magparams, xlist, ylist, t, s

def MapEphem( longn, lat, elev, gmt, twil, interval, steps, initdate, destig, maglimit, altlimit, phasec, forwsca ):
    elements = comephem.ephem()
    kometa = koprclass.comet()
    kometa.LoadComet(destig)
    now = initdate
    
    enddays = (steps-1) * interval + 1
    # 1 7 = 0     # 2 7 = 7     # 4 7 = 21     # 8 7 = 49 
    if enddays > 30:
      enddays == 30
      print('you can plot positions only for 30 days max')
    
    ephem = [  ]
    for i in range(enddays):
       outputele, deltat, peridate = kometa.FindElements(cvt2time(now))
       elements.readdb(outputele)
       date = '{:s}/{:s}/{:s}'.format(str(now.year),str(now.month),str(now.day))

       if phasec == 1 or forwsca == 1:
           elements.compute(date, '00:00', ['comet', 'extend', 'phasec'], longn, lat, float(gmt), float(twil) )
           cmag = elements.mag - elements.phasec * phasec - elements.forwscac * forwsca 
       else:
           elements.compute(date, '00:00', ['comet', 'extend'], longn, lat, float(gmt), float(twil) )
           cmag = elements.mag 
       
       ra, de = comephem.j2000rade( elements.ra, elements.de, date, '00:00' )
       ephem.append( [ now, ra*15., de, cmag, i ] )
       now = now + datetime.timedelta(days = 1)

    return ephem

def EphemGen( longn, lat, elev, gmt, twil, interval, steps, initdate, comets, maglimit, altlimit, phasec, forwsca ):
    comlist = []
    if not comets:
        orb_file = open('./config-files/Elements.txt', 'r', encoding="utf-8")
        ephemlines = orb_file.readlines()
        orb_file.close()
        for line in ephemlines:
            comlist.append(line[102:159].strip())
    else:
        destigs = comets.split(',')
        for i in range(len(destigs)):
            comlist.append(destigs[i].strip())

    #bulharska konstanta blacklist komet#
    try:
      blacklist = ['A','B','C','E','F','G','H','K','L','N','P','Q','R','S','T','U','V','W']
      for blcomet in blacklist:
        comlist.remove('D/1993 F2-'+blcomet+' (Shoemaker-Levy)')
    except:
      pass
    try:
      comlist.remove('1I/`Oumuamua')
    except:
      pass
    try:
      comlist.remove('2I/Borisov')
    except:
      pass
    #bulharska konstanta blacklist komet end#

    cometlist = ''
    elements = comephem.ephem()
    for j in range(len(comlist)):
        if not comets:
          if comlist[j][0:1].isdigit():
              destig = comlist[j].split('/')[0]
          else:
            destig = comlist[j].split(' ')[0]+' '+comlist[j].split(' ')[1]
        else:
            destig = comlist[j]
        kometa = koprclass.comet()
        kometa.LoadComet(destig)

        now = initdate
        desc = 'n'

        for i in range(steps):
           outputele, deltat, peridate = kometa.FindElements(cvt2time(now))
           elements.readdb(outputele)
           date = '{:s}/{:s}/{:s}'.format(str(now.year),str(now.month),str(now.day))
           elements.compute(date, '00:00', ['comet', 'bestt'], longn, lat, float(gmt), float(twil) )
           if elements.btime == 0 and elements.tz == elements.tv:
               break
           elif elements.btime < 0:
               elements.btime = elements.btime + 1.

           time = '{:02d}:{:02d}'.format( int(elements.btime), int((elements.btime - int(elements.btime))*60) )

           if elements.mag < maglimit:
               if phasec == 1 or forwsca == 1:
                   elements.compute(date, time, ['comet', 'extend', 'phasec'], longn, lat, float(gmt), float(twil) )
                   cmag = elements.mag - elements.phasec * phasec - elements.forwscac * forwsca 
               else:
                   elements.compute(date, time, ['comet', 'extend'], longn, lat, float(gmt), float(twil))
                   cmag = elements.mag 

               if elements.alt > altlimit:
                   minalt = round(elements.alt,1)
                   minaz = round(elements.az,1)

                   if desc == 'n':
                       cometlist = cometlist + '\n' + elements.name+'\n' + '  Date        R.A.      Decl.       r      D  Elong m1  Best Time ( A, h ) \n'
                       desc = 'y'
                   ra = '{:0>2d}:{:0>2d}:{:0>2d}'.format( int(elements.ra) , int( (elements.ra-int(elements.ra))*60 ) ,\
                        int((((elements.ra-int(elements.ra))*60)-int((elements.ra-int(elements.ra))*60))*60) ) 
                   decl = '{:+03.0f}:{:0>2d}:{:0>2d}'.format(copysign(int(elements.de),elements.de) , int(abs(elements.de-int(elements.de))*60),\
                          int(abs(((elements.de-int(elements.de))*60)-int((elements.de-int(elements.de))*60))*60) )
                   randomdate = timecvt2( date+' '+time ) + datetime.timedelta( hours = int(gmt) )
                   outpdate = '{:4d}-{:0>2d}-{:0>5.2f}'.format( randomdate.year, randomdate.month, round(randomdate.day + randomdate.hour/24. + randomdate.minute/1440.,2) )
                   cometlist = cometlist + '{:s}  {:>8s}  {:>9s}  {:6.3f} {:6.3f} {:3.0f} {:4.1f} {:s} ( {:5.1f}, {:4.1f})\n'.format(outpdate, \
                               ra, decl, elements.r, elements.dist, elements.elong, cmag, (str(randomdate).split()[1])[:5], minaz, minalt )
           now = now + datetime.timedelta(days = interval)  

    return cometlist

def PlotGen( longn, lat, elev, gmt, twil, now, comets, maglimit, altlimit, phasec, forwsca, initdate ):
    comlist = []
    if not comets:
        orb_file = open('./config-files/Elements.txt', 'r', encoding="utf-8")
        ephemlines = orb_file.readlines()
        orb_file.close()
        for line in ephemlines:
            comlist.append(line[102:159].strip())
        #orb_file = open('./config-files/CustomElements.dat', 'r')
        #ephemlines = orb_file.readlines()
        #orb_file.close()
        #for line in ephemlines:
        #    comlist.append(line[102:159].strip())
    else:
        destigs = comets.split(',')
        for i in range(len(destigs)):
            comlist.append(destigs[i].strip())

    #bulharska konstanta blacklist komet#
    try:
      blacklist = ['A','B','C','E','F','G','H','K','L','N','P','Q','R','S','T','U','V','W']
      for blcomet in blacklist:
        comlist.remove('D/1993 F2-'+blcomet+' (Shoemaker-Levy)')
    except:
      pass
    try:
      comlist.remove('1I/`Oumuamua')
    except:
      pass
    try:
      comlist.remove('2I/Borisov')
    except:
      pass
    #bulharska konstanta blacklist komet end#

    cometlist = []
    elements = comephem.ephem()
    for j in range(len(comlist)):
        if not comets:
          if comlist[j][0:1].isdigit():
            destig = comlist[j].split('/')[0]
          else:
            destig = comlist[j].split(' ')[0]+' '+comlist[j].split(' ')[1]
        else:
          destig = comlist[j]
        kometa = koprclass.comet()
        kometa.LoadComet(destig)
        outputele, deltat, peridate = kometa.FindElements(cvt2time(initdate))
        elements.readdb(outputele)
        
        date = '{:s}/{:s}/{:s}'.format(str(now.year),str(now.month),str(now.day))
        elements.compute(date, '00:00', ['comet', 'bestt'], longn, lat, float(gmt), float(twil) )
        if elements.btime == 0 and elements.tz == elements.tv:
            break
        elif elements.btime < 0:
            elements.btime = elements.btime + 1.

        time = '{:02d}:{:02d}'.format( int(elements.btime), int((elements.btime - int(elements.btime))*60) )

        if elements.mag < maglimit:
            if phasec == 1 or forwsca == 1:
                elements.compute(date, time, ['comet', 'extend', 'phasec'], longn, lat, float(gmt), float(twil) )
                cmag = elements.mag - elements.phasec * phasec - elements.forwscac * forwsca 
            else:
                elements.compute(date, time, ['comet', 'extend'], longn, lat, float(gmt), float(twil))
                cmag = elements.mag 
            minalt = round(elements.alt,1)
            minaz = round(elements.az,1)

            if minalt > altlimit:
              if elements.name[0:1].isdigit():
                cname = elements.name.split('/')[0]#+' ('+str(round(elements.mag,1))+')'
              elif len(elements.name) > 0:
                cname = elements.name.split(' ')[0]+' '+elements.name.split(' ')[1]#+' ('+str(round(elements.mag,1))+')'
              cometlist.append( (cname, elements.ra, elements.de) )

    return cometlist

def DateNow():
    now = datetime.datetime.now()
    return str(now.year)+'/'+str(now.month)+'/'+str(now.day)

def GetRanges(obs_list, date_range, kometa):
    datelist, deltatlist, rlist = [], [], []
    for key in list(obs_list.keys()): # define ranges
      for i in range(4):
        if obs_list[key][i]: 
          for line in obs_list[key][i]:
            obs = koprclass.icqcode()
            obs.decode_icq(line)
            datelist.append(timecvt(obs.date))
            if date_range != 'Date':
              eline, deltadate, peridate = kometa.FindElements(obs.date)
              deltatlist.append(deltadate)
              elements = comephem.ephem()
              elements.readdb(eline)
              elements.compute(obs.date,'00:00', ['comet'])
              rlist.append(elements.r)

    if date_range == 'Date': 
        strfrom = str(cvt2time(min(datelist) - datetime.timedelta(days = 1)))
        strto = str(cvt2time(max(datelist) + datetime.timedelta(days = 1)))
    elif date_range == 'Days from T':
        strfrom = str( min(deltatlist) )
        strto = str( max(deltatlist) )
    elif date_range == 'r':
        strfrom = str('{:6.3f}'.format(round(min(rlist) - 0.001,3)) )
        strto = str('{:6.3f}'.format(round(max(rlist) + 0.001,3)) )
    elif date_range == 'Log(r)':
        strfrom = str('{:6.4f}'.format(round(log10(min(rlist) - 0.001),4)) )
        strto = str('{:6.4f}'.format(round(log10(max(rlist) + 0.001),4)) )

    return strfrom, strto

def FilterObs(settings, observers, perihelions, types, obs_list, kometa, rangefrom, rangeto, date_range, compare = 0 ):
    alwd_obs, alwd_peri, alwd_types = [], [], [] 

    for line in observers:
      if line[1] == 'o' :
         alwd_obs.append( line[0] )
    for line in perihelions:
      if line[1] == 'o' :
         alwd_peri.append( line[0] )
    for line in types:
      if line[1] == 'o' :
         alwd_types.append( line[0] )

    elements = comephem.ephem()

    if date_range == 'Date':
      range_from, range_to = timecvt(rangefrom), timecvt(rangeto)
    elif date_range == 'Days from T':
      range_from, range_to = float(rangefrom), float(rangeto)
    elif date_range == 'r':
      range_from, range_to = float(rangefrom), float(rangeto)
    elif date_range == 'Log(r)':
      range_from, range_to = float(rangefrom), float(rangeto)

    finallist, newlist = [], []

    for key in list(obs_list.keys()): # define ranges
      if key in alwd_peri:
        for i in range(4):
          if obs_list[key][i]: 
            for line in obs_list[key][i]:
              obs = koprclass.icqcode()
              obs.decode_icq(line)
              eline, deltadate, peridate = kometa.FindElements(obs.date)
              elements.readdb(eline)
              if date_range == 'Date':
                if range_from < timecvt(obs.date) < range_to:
                  if not ((settings[0] == 1 and obs.uncsign == ':') or (settings[1] == 1 and obs.negsign == '[')) and (obs.obscode.strip() in alwd_obs) and (obs.otype in alwd_types):
                    if settings[2] == 1: 
                      elements.compute(obs.date,'00:00', ['comet'])
                      if (elements.mag - settings[3]) < obs.mag < (elements.mag + settings[3]):
                        finallist.append(line)
                      else:
                        newlist.append(line)
                    else:
                      finallist.append(line)
                  elif compare == 1:
                    newlist.append(line)

              elif date_range == 'Days from T':
                if range_from < deltadate < range_to:
                  if not ((settings[0] == 1 and obs.uncsign == ':') or (settings[1] == 1 and obs.negsign == '[')) and (obs.obscode.strip() in alwd_obs) and (obs.otype in alwd_types):
                    if settings[2] == 1: 
                      elements.compute(obs.date,'00:00', ['comet'])
                      if (elements.mag - settings[3]) < obs.mag < (elements.mag + settings[3]):
                        finallist.append(line)
                      else:
                        newlist.append(line)
                    else:
                      finallist.append(line)
                  elif compare == 1:
                    newlist.append(line)

              elif date_range == 'r':
                elements.compute(obs.date,'00:00', ['comet'])
                if range_from < elements.r < range_to:
                  if not ((settings[0] == 1 and obs.uncsign == ':') or (settings[1] == 1 and obs.negsign == '[')) and (obs.obscode.strip() in alwd_obs) and (obs.otype in alwd_types):
                    if settings[2] == 1: 
                      if (elements.mag - settings[3]) < obs.mag < (elements.mag + settings[3]):
                        finallist.append(line)
                      else:
                        newlist.append(line)
                    else:
                      finallist.append(line)
                  elif compare == 1:
                    newlist.append(line)

              elif date_range == 'Log(r)':
                elements.compute(obs.date,'00:00', ['comet'])
                if range_from < log10(elements.r) < range_to:
                  if not ((settings[0] == 1 and obs.uncsign == ':') or (settings[1] == 1 and obs.negsign == '[')) and (obs.obscode.strip() in alwd_obs) and (obs.otype in alwd_types):
                    if settings[2] == 1: 
                      if (elements.mag - settings[3]) < obs.mag < (elements.mag + settings[3]):
                        finallist.append(line)
                      else:
                        newlist.append(line)
                    else:
                      finallist.append(line)
                  elif compare == 1:
                    newlist.append(line)
        
    return finallist, newlist

def GenerateLists(settings, observers, perihelions, types, finallist, kometa, rangefrom, rangeto, date_range):
    elements = comephem.ephem()
    if date_range == 'Date':
      range_from, range_to = timecvt(rangefrom), timecvt(rangeto)
    elif date_range == 'Days from T':
      range_from, range_to = float(rangefrom), float(rangeto)
    elif date_range == 'r':
      range_from, range_to = float(rangefrom), float(rangeto)
    elif date_range == 'Log(r)':
      range_from, range_to = float(rangefrom), float(rangeto)
    #definitivni vyrazeni
    newlist = []
    for line in obs_list:
      obs = koprclass.icqcode()
      obs.decode_icq(line)
      eline, deltadate, peridate = kometa.FindElements(obs.date)
      if peridate[0:4] in alwd_peri:
        if date_range == 'Date':
          if range_from < timecvt(obs.date) < range_to:
            newlist.append(line)
        elif date_range == 'Days from T':
          if range_from < deltadate < range_to:
            newlist.append(line)
        elif date_range == 'r':
          elements.readdb(eline)
          elements.compute(obs.date,'00:00', ['comet'])
          if range_from < elements.r < range_to:
            newlist.append(line)
        elif date_range == 'Log(r)':
          elements.readdb(eline)
          elements.compute(obs.date,'00:00', ['comet'])
          if range_from < log10(elements.r) < range_to:
            newlist.append(line)
    
    finallist = []
    #compare, perasy or nothing
    for line in newlist:
      obs = koprclass.icqcode()
      obs.decode_icq(line)
      if not ((settings[0] == 1 and obs.uncsign == ':') or (settings[1] == 1 and obs.negsign == '[')) and (obs.obscode.strip() in alwd_obs) and (obs.otype in alwd_types):
        eline, deltadate, peridate = kometa.FindElements(obs.date)
        if settings[2] == 1: 
          elements.readdb(eline)
          elements.compute(obs.date,'00:00', ['comet'])
          if (elements.mag - settings[3]) < obs.mag < (elements.mag + settings[3]):
            finallist.append(line)
        else:
          finallist.append(line)

    return finallist

def SettingsUpdate(settings):
    f = open('./config-files/settings.dat', 'w', encoding="utf-8")
    for item in settings:
      f.write(';'.join(item)+'\n')
    f.close()

def SettingsRead():
    settings = []
    f = open('./config-files/settings.dat', 'r', encoding="utf-8")
    content = f.read().split('\n')
    for item in content[:-1]:
      settings.append(item.split(';'))
    f.close()
    return settings

